@@ -77,8 +77,7 @@ def test_04_indexed_ended_by_exception():
 def test_05_select_limit():
     setupIter()
     assert len(list(IterTest.select(limit=2))) == 2
-    raises(AssertionError, IterTest.select(limit=2).distinct)
-    raises(AssertionError, IterTest.select(limit=2).clone, start=1)
+    raises(AssertionError, IterTest.select(limit=2).count)
 
 def test_06_like():
     setupIter()
@@ -86,6 +85,7 @@ def test_06_like():
     assert len(list(IterTest.select(IterTest.q.name.endswith('a')))) == 1
     assert len(list(IterTest.select(IterTest.q.name.contains('a')))) == 1
     assert len(list(IterTest.select(IterTest.q.name.contains(func.lower('A'))))) == 1
+    assert len(list(IterTest.select(IterTest.q.name.contains("a'b")))) == 0
 
 def test_select_getOne():
     setupClass(IterTest)
@@ -184,8 +184,3 @@ def test_select_RLIKE():
     assert IterTest.select(RLIKE(IterTest.q.name, "^sqlb.*$")).count() == 1
     assert IterTest.select(RLIKE(IterTest.q.name, "^sqlb.*$")).count() == 1
     assert IterTest.select(RLIKE(IterTest.q.name, "^sqlx.*$")).count() == 0
-
-def test_bool_SelectResults():
-    setupClass(IterTest)
-
-    raises(NotImplementedError, bool, IterTest.select())
